// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

// Copyright (C) 2012	stfour
// Copyright (C) 2012	JoostinOnline

#define GRLIB

#include <stdarg.h>
#include <stdlib.h>
#include <wiiuse/wpad.h>
#include <ogc/lwp_watchdog.h>
#include "../debug.h"
#include "grlib.h"

GRRLIB_texImg *popPushTex = NULL;
GRRLIB_texImg *redrawTex = NULL;
GRRLIB_ttfFont *myFont;

s_grlibSettings grlibSettings;

s_ir grlib_irPos;

// WPADShutdownCallback
static void wmpower_cb( s32 chan )
	{
	grlibSettings.wiiswitch_poweroff = true;
	return;
	}

static void power_cb()
	{
	grlibSettings.wiiswitch_poweroff = true;
	}

static void reset_cb()
	{
	grlibSettings.wiiswitch_reset = true;
	}

void grlib_Controllers (bool enable)
	{
	if (enable)
		{
		PAD_Init();
		WPAD_Init();
		WPAD_SetDataFormat(WPAD_CHAN_0, WPAD_FMT_BTNS_ACC_IR);
		WPAD_SetVRes(0, 640, 480);
		WPAD_SetPowerButtonCallback( wmpower_cb );
		}
	else
		{
		u8 i;

		for (i = 0; i < WPAD_MAX_WIIMOTES; i++)
			{
			if(WPAD_Probe(i,0) < 0) continue;

			WPAD_Flush(i);
			WPAD_Disconnect(i);
			}

		WPAD_Shutdown();
		PAD_Reset(0xf0000000);
		}
	}

void grlib_Init (void)
	{
	memset (&grlibSettings, 0, sizeof (s_grlibSettings));
	grlibSettings.wiiswitch_poweroff = false;
	grlibSettings.wiiswitch_reset = false;

	// Initialise the Wiimotes & Gamecube
	PAD_Init();
	WPAD_Init();
	#ifndef DOLPHIN
	// This needs to be disabled when running in Dolphin, otherwise a virtual wiimote
	// will timeout even when active.  This bug may only exist in certain revisions.
	WPAD_SetIdleTimeout(60 * 1.5); // Remote will turn off after 1.5 minutes of inactivity.
	#endif
	WPAD_SetDataFormat(WPAD_CHAN_0, WPAD_FMT_BTNS_ACC_IR);

	VIDEO_Init();
	VIDEO_SetBlack(true);  // Disable video output during initialisation
	VIDEO_WaitVSync();
	VIDEO_WaitVSync();

	GRRLIB_Init(1,0);

	WPAD_SetVRes(0, 640, 480);
	WPAD_SetPowerButtonCallback( wmpower_cb );

	SYS_SetPowerCallback (power_cb);
	SYS_SetResetCallback (reset_cb);

	redrawTex = GRRLIB_CreateEmptyTexture (rmode->fbWidth, rmode->efbHeight);
	popPushTex = GRRLIB_CreateEmptyTexture (rmode->fbWidth, rmode->efbHeight);

	grlibSettings.RedrawCallback = NULL;

	grlibSettings.pointer[0] = NULL;
	grlibSettings.pointer[1] = NULL;
	grlibSettings.pointer[2] = NULL;
	grlibSettings.pointer[3] = NULL;
	}

void grlib_Exit (void) {
	GRRLIB_FreeTexture (redrawTex);
	GRRLIB_FreeTexture (popPushTex);
	Debug("FreeTexture Passed");
	if (myFont != NULL) GRRLIB_FreeTTF(myFont);
	Debug("myFont Check and FreeTTF Passed");
	if (!grlibSettings.doNotCall_GRRLIB_Exit)
	GRRLIB_Exit ();
	else
		GRRLIB_ExitLight ();
	Debug("GRRLIB_Exit Passed");
}

void grlib_SetRedrawCallback (GRLIB_RedrawCallback cbRedraw, GRLIB_RedrawCallback cbOverlay)
	{
	grlibSettings.RedrawCallback = cbRedraw;
	grlibSettings.OverlayCallback = cbOverlay;
	}

void grlib_SetWiimotePointer (GRRLIB_texImg *pointer, int num)
	{
	grlibSettings.pointer[num] = pointer;
	}

int grlib_GetScreenW (void)
	{
	return rmode->fbWidth;
	}

int grlib_GetScreenH (void)
	{
	return rmode->efbHeight;
	}

void grlib_PushScreen (void)
	{
	GRRLIB_Screen2Texture (0, 0, popPushTex, 0);
	//grlib_PopScreen ();
	}

void grlib_PopScreen (void)
	{
	bool al = GRRLIB_GetAntiAliasing ();
	GRRLIB_SetAntiAliasing (0);
	GRRLIB_DrawImg(0, 0, popPushTex, 0, 1, 1, RGBA(255, 255, 255, 255) );
	GRRLIB_SetAntiAliasing (al);
	}

GRRLIB_texImg * grlib_CreateTexFromTex (GRRLIB_texImg *source)
	{
	GRRLIB_texImg * tex;
	GRRLIB_SetAntiAliasing (FALSE);
	if (source == NULL)
		source = redrawTex; //return NULL;

	tex = GRRLIB_CreateEmptyTexture (source->w, source->h);
	if (tex == NULL) return NULL;

	memcpy (tex->data, source->data, source->w * source->h * 4);
	GRRLIB_FlushTex (tex);
	return tex;
	}

// Draw the latest buffered popPushTex... but before do that discard all previous texture
void grlib_DrawScreenTex (GRRLIB_texImg * tex)
	{
	bool al = GRRLIB_GetAntiAliasing ();
	GRRLIB_SetAntiAliasing (FALSE);
	GRRLIB_DrawImg (0, 0, tex , 0, 1, 1, RGBA(255, 255, 255, 255) );
	GRRLIB_SetAntiAliasing (al);
	}

// Call the current registered redraw function
void grlib_Redraw (void)
	{
	if (grlibSettings.RedrawCallback != NULL) grlibSettings.RedrawCallback();
	}

void grlib_Render (void)
	{
	// Sometimes same fb are skipped, I do not know why,
	// but copyng display to tex and redrawing it solves without big effort
	// and loss of performance.
	// In continuos update mode, doesn't heppen, so GRRLIB_Render can be called.
	GRRLIB_Screen2Texture (0, 0, redrawTex, 1);
	grlib_DrawScreenTex (redrawTex);
	GRRLIB_Render ();
	}

void grlib_Text (f32 xpos, f32 ypos, u8 align, u32 color, char *text)
	{
	f32 xoff = xpos;
	ypos --;

	if (align == GRLIB_ALIGNCENTER)
		xoff -= grlib_GetFontMetrics (text, NULL, NULL) / 2;

	if (align == GRLIB_ALIGNRIGHT)
		xoff -= grlib_GetFontMetrics (text, NULL, NULL);
	if (grlibSettings.fontMode == GRLIB_FONTMODE_TTF) {
		if (grlibSettings.fontReverse)
			GRRLIB_PrintfTTF(xoff, ypos, myFont, text, FONTSIZE, 0x0 | 0xFF);
		else
			GRRLIB_PrintfTTF(xoff, ypos, myFont, text, FONTSIZE, 0xFFFFFFFF | 0xFF);
	}
	return;
	}

void grlib_printf (const f32 xpos, const f32 ypos, const u8 align, const u32 color, const char *text, ...)
	{
	char  tmp[1024];

	va_list argp;
	va_start(argp, text);
	vsprintf(tmp, text, argp);
	va_end(argp);

	grlib_Text (xpos, ypos, align, color, tmp);
	}

void grlib_DrawSquare ( s_grlibobj *b )
	{
	GRRLIB_Rectangle ( b->x1, b->y1, b->x2-b->x1, b->y2-b->y1, b->bcolor, 1 );
	GRRLIB_Rectangle ( b->x1, b->y1, b->x2-b->x1, b->y2-b->y1, b->color, 0 );
	}

void grlib_DrawEmptySquare ( s_grlibobj *b )
	{
	GRRLIB_Rectangle ( b->x1, b->y1, b->x2-b->x1, b->y2-b->y1, b->color, 0 );
	}

void grlib_DrawBoldEmptySquare ( s_grlibobj *b )
	{
	GRRLIB_Rectangle ( b->x1, b->y1, b->x2-b->x1, b->y2-b->y1, b->color, 0 );
	GRRLIB_Rectangle ( b->x1-1, b->y1-1, b->x2-b->x1+2, b->y2-b->y1+2, b->color, 0 );
	GRRLIB_Rectangle ( b->x1-2, b->y1-2, b->x2-b->x1+4, b->y2-b->y1+4, b->color, 0 );
	}

// Set the font to be used for the next txt operation
void grlib_SetFontBMF (GRRLIB_bytemapFont *bmf)
	{
	grlibSettings.fontBMF = bmf;
	grlibSettings.fontMode = GRLIB_FONTMODE_BMF;
	}

// return width and height of the font. height/width can be NULL... return is the width
int grlib_GetFontMetrics (const char *text, int *width, int *height)
	{
	f32 xoff = 0;

	if (grlibSettings.fontMode == GRLIB_FONTMODE_TTF)
		{
		xoff = GRRLIB_WidthTTF(myFont, text, FONTSIZE);
		if (height) *height = 9; // Should be measured
		if (width) *width = xoff;
		return xoff;
		}
	return 0;
	}

void grlib_ComputeObjectMeanPoint (s_grlibobj *go)
	{
	go->xm = go->x1 + (go->x2 - go->x1) / 2;
	go->ym = go->y1 + (go->y2 - go->y1) / 2;
	}


// Draw a window centered in the screen... return the y offset of the title.. go is the bounding rect
int grlib_DrawCenteredWindow (char * title, int w, int h, bool grayoutBackground, s_grlibobj *go)
	{
	s_grlibobj goWindow;
	int halfx, halfy;
	int th, tw;
	char *t;
	int l = 0;

	l = strlen (title);
	t = malloc (l+1);
	strcpy (t, title);

	if (l > 0)
		{
		l--;

		do
			{
			grlib_GetFontMetrics (t, &tw, &th);
			if (tw > w)
				{
				t[l] = 0;
				l--;
				}
			else
				break;

			if (l == 0) break;
			}
		while (TRUE);
		}

	// Grayout background
	if (grayoutBackground)
		{
		goWindow.x1 = -100;
		goWindow.y1 = -100;
		goWindow.x2 = grlib_GetScreenW()+100;
		goWindow.y2 = grlib_GetScreenH()+100;
		goWindow.color = RGBA(0,0,0,128);
		goWindow.bcolor = RGBA(0,0,0,128);
		grlib_DrawSquare (&goWindow);
		}

	halfx = rmode->fbWidth / 2;
	halfy = rmode->efbHeight / 2;

	goWindow.x1 = halfx - (w / 2);
	goWindow.x2 = halfx + (w / 2);

	goWindow.y1 = halfy - (h / 2);
	goWindow.y2 = halfy + (h / 2);

	goWindow.bcolor = RGBA (32, 32, 32, 192);
	goWindow.color = RGBA (192, 192, 192, 255);

	if (grlibSettings.theme.enabled)
		{
		grlib_DrawSquareThemed (&goWindow, grlibSettings.theme.texWindow, grlibSettings.theme.texWindowBk, grlibSettings.theme.windowMagX, grlibSettings.theme.windowMagY, DSTF_BKFILLBORDER);
		}
	else
		{
		grlib_DrawSquare (&goWindow);
		grlib_DrawBoldEmptySquare (&goWindow);
		}

	grlib_Text (halfx, goWindow.y1 + 10, GRLIB_ALIGNCENTER, 0, t);

	if (go)
		{
		go->x1 = goWindow.x1;
		go->x2 = goWindow.x2;
		go->y1 = goWindow.y1;
		go->y2 = goWindow.y2;

		grlib_ComputeObjectMeanPoint(go);
		}

	return 0;
	}

void grlib_DrawImgCenter (int x, int y, int w, int h, GRRLIB_texImg * tex, f32 angle, u32 color)
	{
	f32 zx, zy;

	if (!tex) return;

	zx = (f32)w / tex->w;
	zy = (f32)h / tex->h;

	GRRLIB_SetHandle (tex, tex->w / 2, tex->h / 2);
	//GRRLIB_SetHandle (tex, 0,0);

	x -= tex->w/2;
	y -= tex->w/2;

	GRRLIB_DrawImg (x, y, tex, angle, zx, zy, color );
	}

void grlib_DrawImg (int x, int y, int w, int h, GRRLIB_texImg * tex, f32 angle, u32 color)
	{
	f32 zx, zy;

	if (!tex) return;

	zx = (f32)w / tex->w;
	zy = (f32)h / tex->h;

	GRRLIB_DrawImg (x, y, tex, angle, zx, zy, color );
	}

void grlib_DrawTile (int x, int y, int w, int h, GRRLIB_texImg * tex, f32 angle, u32 color, int frame)
	{
	f32 zx, zy;

	if (!tex) return;

	zx = (f32)w / tex->tilew;
	zy = (f32)h / tex->tileh;

	GRRLIB_DrawTile (x, y, tex, angle, zx, zy, color, frame);
	}

void grlib_DrawPart (int x, int y, int w, int h, int tx, int ty, int tw, int th, GRRLIB_texImg * tex, f32 angle, u32 color)
	{
	f32 zx, zy;

	if (!tex) return;

	zx = (f32)w / (f32)tw;
	zy = (f32)h / (f32)th;

	GRRLIB_DrawPart (x, y, tx, ty, tw, th, tex, angle, zx, zy, color);
	}


void grlib_DrawIRCursor (void)
	{
	ir_t ir;
	static u8 alphadec = 255;
	static s16 alpha = 0; // Start with cursor hidden
	static u32 cursorActivity = 0;

	static u32 startms = 0;
	u32 ms = ticks_to_millisecs(gettime());

	WPAD_IR (0, &ir);

	if (ms > startms)
		{
		if (cursorActivity == grlibSettings.cursorActivity)
			{
			alpha -= alphadec;
			if (alpha < 0) alpha = 0;
			}
		else
			{
			alpha = 255;
			alphadec = 5;
			cursorActivity = grlibSettings.cursorActivity;
			}

		startms = ms+100;
		}

	if (ir.valid)
		{
		grlib_irPos.x = ir.x;
		grlib_irPos.y = ir.y;
		grlib_irPos.valid = 1;
		alpha = 255;
		}
	else
		{
		grlib_irPos.valid = 0;
		}

	GRRLIB_DrawImg( grlib_irPos.x,
					grlib_irPos.y,
					grlibSettings.pointer[0], 0, 1, 1, RGBA(255, 255, 255, alpha) );
	}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// This function scan for controllers and return the right common value
// it also support analog sticks for move cursor on the screen

#define MAXG 6
int grlib_GetUserInput (void)
	{
	u32  wbtn, gcbtn, cbtn;
	s8  gcX, gcY;
	int nX, nY;
	int cX, cY;

	static float g[MAXG];
	static int gidx = -1;
	static u32 gestureDisable = 0;

	u32 ms = ticks_to_millisecs(gettime());
	u32 mstout = ms+200;

	struct expansion_t e; //nunchuk

	if (gidx == -1)
		{
		memset (&g, 0, sizeof(g));
		gidx = 0;
		}

	WPAD_ScanPads();  // Scan the Wiimotes
	wbtn = WPAD_ButtonsDown(0);

	if (grlibSettings.usesGestures && ms > gestureDisable)
		{
		WPADData *wp = WPAD_Data (0);
		g[gidx] = wp->gforce.x;
		/*
		int gidx2 = gidx - 10;
		int gidx3 = gidx - 20;
		if (gidx2 < 0) gidx2 = (MAXG-1) - gidx2;
		if (gidx3 < 0) gidx3 = (MAXG-1) - gidx3;

		if (g[gidx2] < -2.0 && g[gidx] > -1.5 && g[gidx3] > -1.5)
			{
			memset (&g, 0, sizeof(g));
			return WPAD_BUTTON_MINUS;
			//Debug ("left");
			}
		if (g[gidx2] > 2.0 && g[gidx] < 1.5 && g[gidx3] < 1.5)
			{
			memset (&g, 0, sizeof(g));
			return WPAD_BUTTON_PLUS;
			//Debug ("right");
			}
		*/
		int i;
		float mean = 0;
		for (i = 0; i < MAXG; i++)
			mean+=g[i];
		mean /= (float) MAXG;

		// gprintf ("%.2f\n", mean);

		if (mean < -1.5)
			{
			memset (&g, 0, sizeof(g));
			gestureDisable = ms+1000;
			return WPAD_BUTTON_MINUS;
			}
		if (mean > 1.5)
			{
			memset (&g, 0, sizeof(g));
			gestureDisable = ms+1000;
			return WPAD_BUTTON_PLUS;
			}

		if (++gidx >= MAXG) gidx = 0;
		}

	//Debug ("wm = %.1f,%.1f,%.1f", wp->gforce.x,wp->gforce.y,wp->gforce.z);
	//Debug ("wm = %.1f,%.1f,%.1f", wp->orient.roll,wp->orient.pitch,wp->orient.yaw);


	WPAD_Expansion( 0, &e );

	if (e.type != WPAD_EXP_NUNCHUK)
		{
		nX = 0;
		nY = 0;
		}
	else
		{
		nX = e.nunchuk.js.pos.x - e.nunchuk.js.center.x;
		nY = e.nunchuk.js.pos.y - e.nunchuk.js.center.y;
		}

	if (e.type != WPAD_EXP_CLASSIC)
		{
		cX = 0;
		cY = 0;
		cbtn = 0;
		}
	else
		{
		cX = e.classic.ljs.pos.x - e.classic.ljs.center.x;
		cY = e.classic.ljs.pos.y - e.classic.ljs.center.y;
		cbtn = e.classic.btns;
		}

	PAD_ScanPads();
	gcX = PAD_StickX(0);
	gcY = PAD_StickY(0);
	gcbtn = PAD_ButtonsDown(0);

	// sticks
	if (abs (nX) > 10) {grlib_irPos.x += (nX / 16); grlibSettings.cursorActivity++;}
	if (abs (nY) > 10) {grlib_irPos.y -= (nY / 16); grlibSettings.cursorActivity++;}

	if (abs (gcX) > 10) {grlib_irPos.x += (gcX / 16); grlibSettings.cursorActivity++;}
	if (abs (gcY) > 10) {grlib_irPos.y -= (gcY / 16); grlibSettings.cursorActivity++;}

	if (abs (cX) > 10) {grlib_irPos.x += (cX / 4); grlibSettings.cursorActivity++;}
	if (abs (cY) > 10) {grlib_irPos.y -= (cY / 4); grlibSettings.cursorActivity++;}

	// Check limits
	if (grlib_irPos.x < 0) grlib_irPos.x = 0;
	if (grlib_irPos.x > 640) grlib_irPos.x = 640;

	if (grlib_irPos.y < 0) grlib_irPos.y = 0;
	if (grlib_irPos.y > 480) grlib_irPos.y = 480;

	// As usual wiimotes will have priority
	if (wbtn)
		{
		grlibSettings.buttonActivity ++;
		// Wait until button is released

		while (WPAD_ButtonsDown(0) && ticks_to_millisecs(gettime()) < mstout) WPAD_ScanPads();
		return wbtn;
		}

	// Then gc
	if (gcbtn)
		{
		grlibSettings.buttonActivity ++;

		// Wait until button is released

		while (PAD_ButtonsDown(0) && ticks_to_millisecs(gettime()) < mstout) PAD_ScanPads();

		// Convert to wiimote values
		if (gcbtn & PAD_TRIGGER_R) return WPAD_BUTTON_PLUS;
		if (gcbtn & PAD_TRIGGER_L) return WPAD_BUTTON_MINUS;

		if (gcbtn & PAD_BUTTON_A) return WPAD_BUTTON_A;
		if (gcbtn & PAD_BUTTON_B) return WPAD_BUTTON_B;
		if (gcbtn & PAD_BUTTON_X) return WPAD_BUTTON_1;
		if (gcbtn & PAD_BUTTON_Y) return WPAD_BUTTON_2;
		if (gcbtn & PAD_BUTTON_MENU) return WPAD_BUTTON_HOME;
		if (gcbtn & PAD_BUTTON_UP) return WPAD_BUTTON_UP;
		if (gcbtn & PAD_BUTTON_LEFT) return WPAD_BUTTON_LEFT;
		if (gcbtn & PAD_BUTTON_DOWN) return WPAD_BUTTON_DOWN;
		if (gcbtn & PAD_BUTTON_RIGHT) return WPAD_BUTTON_RIGHT;
		}

	// Classic
	if (cbtn)
		{
		grlibSettings.buttonActivity ++;

		while (e.classic.btns && ticks_to_millisecs(gettime()) < mstout)
			{
			WPAD_ScanPads();  // Scan the Wiimotes
			WPAD_Expansion( 0, &e );
			}

		// Convert to wiimote values
		if (cbtn & CLASSIC_CTRL_BUTTON_ZR) return WPAD_BUTTON_PLUS;
		if (cbtn & CLASSIC_CTRL_BUTTON_ZL) return WPAD_BUTTON_MINUS;

		if (cbtn & CLASSIC_CTRL_BUTTON_PLUS) return WPAD_BUTTON_PLUS;
		if (cbtn & CLASSIC_CTRL_BUTTON_MINUS) return WPAD_BUTTON_MINUS;

		if (cbtn & CLASSIC_CTRL_BUTTON_A) return WPAD_BUTTON_A;
		if (cbtn & CLASSIC_CTRL_BUTTON_B) return WPAD_BUTTON_B;
		if (cbtn & CLASSIC_CTRL_BUTTON_X) return WPAD_BUTTON_1;
		if (cbtn & CLASSIC_CTRL_BUTTON_Y) return WPAD_BUTTON_2;
		if (cbtn & CLASSIC_CTRL_BUTTON_HOME) return WPAD_BUTTON_HOME;
		}


	return 0;
	}


void grlib_MagObject ( s_grlibobj *bt, s_grlibobj *bs, f32 magx, f32 magy)
	{
	memmove (bt, bs, sizeof(s_grlibobj));

	bt->x1 -= magx;
	bt->y1 -= magy;
	bt->x2 += magx;
	bt->y2 += magy;
	}

